/**
* Copyright Zendesk, Inc.
*
* Use of this source code is governed under the Apache License, Version 2.0
* found at http://www.apache.org/licenses/LICENSE-2.0.
*/
import React__default, { forwardRef, useState, useContext } from 'react';
import PropTypes from 'prop-types';
import { ThemeContext } from 'styled-components';
import { usePagination } from '@zendeskgarden/container-pagination';
import { getControlledValue } from '@zendeskgarden/container-utilities';
import { StyledPagination } from '../../styled/Pagination/StyledPagination.js';
import '../../styled/Pagination/StyledPage.js';
import '../../styled/CursorPagination/StyledCursorPagination.js';
import '../../styled/CursorPagination/StyledCursor.js';
import '../../styled/CursorPagination/StyledIcon.js';
import '../../styled/Pagination/StyledGap.js';
import '../../styled/Pagination/StyledNavigation.js';
import { Previous } from './components/Previous.js';
import { Next } from './components/Next.js';
import { Page } from './components/Page.js';
import { Gap } from './components/Gap.js';

const PREVIOUS_KEY = 'previous';
const NEXT_KEY = 'next';
const Pagination = forwardRef((_ref, ref) => {
  let {
    currentPage: controlledCurrentPage,
    transformPageProps,
    totalPages,
    pagePadding,
    pageGap,
    onChange,
    ...otherProps
  } = _ref;
  const [focusedItem, setFocusedItem] = useState();
  const [internalCurrentPage, setInternalCurrentPage] = useState(1);
  const currentPage = getControlledValue(controlledCurrentPage, internalCurrentPage);
  const theme = useContext(ThemeContext);
  const {
    getContainerProps,
    getPageProps,
    getPreviousPageProps,
    getNextPageProps
  } = usePagination({
    rtl: theme.rtl,
    focusedItem,
    selectedItem: currentPage,
    onFocus: item => {
      setFocusedItem(item);
    },
    onSelect: item => {
      let updatedCurrentPage = item;
      let updatedFocusedKey = focusedItem;
      if (updatedCurrentPage === PREVIOUS_KEY && currentPage > 1) {
        updatedCurrentPage = currentPage - 1;
        if (updatedCurrentPage === 1 && focusedItem === PREVIOUS_KEY) {
          updatedFocusedKey = 1;
        }
      } else if (updatedCurrentPage === NEXT_KEY && currentPage < totalPages) {
        updatedCurrentPage = currentPage + 1;
        if (updatedCurrentPage === totalPages && updatedFocusedKey === NEXT_KEY) {
          updatedFocusedKey = totalPages;
        }
      }
      if (onChange && updatedCurrentPage !== undefined) {
        onChange(updatedCurrentPage);
      }
      setFocusedItem(updatedFocusedKey);
      setInternalCurrentPage(updatedCurrentPage);
    }
  });
  const getTransformedProps = function (pageType) {
    let props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    let pageNumber = arguments.length > 2 ? arguments[2] : undefined;
    if (transformPageProps) {
      return transformPageProps(pageType, props, pageNumber);
    }
    return props;
  };
  const renderPreviousPage = () => {
    const isFirstPageSelected = totalPages > 0 && currentPage === 1;
    const focusRef = React__default.createRef();
    const props = isFirstPageSelected ?
    {
      hidden: true
    } : {
      ...getPreviousPageProps({
        'aria-label': '',
        role: null,
        item: PREVIOUS_KEY,
        focusRef
      }),
      'aria-label': undefined
    };
    return React__default.createElement(Previous, Object.assign({
      isFocused: focusedItem === PREVIOUS_KEY
    }, getTransformedProps('previous', props), {
      ref: focusRef
    }));
  };
  const renderNextPage = () => {
    const isLastPageSelected = currentPage === totalPages;
    const focusRef = React__default.createRef();
    const props = isLastPageSelected ?
    {
      hidden: true
    } : {
      ...getNextPageProps({
        'aria-label': '',
        role: null,
        item: NEXT_KEY,
        focusRef
      }),
      'aria-label': undefined
    };
    return React__default.createElement(Next, Object.assign({
      isFocused: focusedItem === NEXT_KEY
    }, getTransformedProps('next', props), {
      ref: focusRef
    }));
  };
  const createGap = pageIndex => React__default.createElement(Gap, Object.assign({
    key: `gap-${pageIndex}`
  }, getTransformedProps('gap')));
  const createPage = pageIndex => {
    const focusRef = React__default.createRef();
    const props = {
      ...getPageProps({
        'aria-label': '',
        role: null,
        item: pageIndex,
        focusRef
      }),
      'aria-label': undefined,
      title: pageIndex
    };
    return React__default.createElement(Page, Object.assign({
      key: pageIndex
    }, getTransformedProps('page', props, pageIndex), {
      ref: focusRef
    }), pageIndex);
  };
  const renderPages = () => {
    const pages = [];
    const PADDING = pagePadding;
    const GAP = pageGap;
    for (let pageIndex = 1; pageIndex <= totalPages; pageIndex++) {
      if (pageIndex === currentPage || pageIndex < GAP || pageIndex > totalPages - GAP + 1) {
        pages.push(createPage(pageIndex));
        continue;
      }
      let minimum;
      let maximum;
      if (currentPage <= GAP + PADDING) {
        minimum = GAP + 1;
        maximum = minimum + PADDING * 2;
      } else if (currentPage >= totalPages - GAP - PADDING) {
        maximum = totalPages - GAP;
        minimum = maximum - PADDING * 2;
      } else {
        minimum = currentPage - PADDING;
        maximum = currentPage + PADDING;
      }
      if (pageIndex >= minimum && pageIndex <= currentPage || pageIndex >= currentPage && pageIndex <= maximum) {
        pages.push(createPage(pageIndex));
        continue;
      }
      if (pageIndex === GAP) {
        if (minimum > GAP + 1 && currentPage > GAP + PADDING + 1) {
          pages.push(createGap(pageIndex));
        } else {
          pages.push(createPage(pageIndex));
        }
        continue;
      }
      if (pageIndex === totalPages - GAP + 1) {
        if (maximum < totalPages - GAP && currentPage < totalPages - GAP - PADDING) {
          pages.push(createGap(pageIndex));
        } else {
          pages.push(createPage(pageIndex));
        }
        continue;
      }
    }
    return pages;
  };
  return React__default.createElement(StyledPagination, Object.assign({}, getContainerProps({
    role: null
  }), otherProps, {
    ref: ref
  }), renderPreviousPage(), totalPages > 0 && renderPages(), renderNextPage());
});
Pagination.propTypes = {
  currentPage: PropTypes.number.isRequired,
  totalPages: PropTypes.number.isRequired,
  pagePadding: PropTypes.number,
  pageGap: PropTypes.number,
  onChange: PropTypes.func,
  transformPageProps: PropTypes.func
};
Pagination.defaultProps = {
  pagePadding: 2,
  pageGap: 2
};
Pagination.displayName = 'Pagination';

export { Pagination };
